/*
Copyright (C) 2012 Taisuke Kobayashi
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.util.ArrayList;
import java.util.Arrays;

public class ClientList {

  static final String DELETE_FLAG_TRUE = "t", DELETE_FLAG_FALSE = "f";
  static final String FIELD_SEP = ",", INFO_SEP = " ", TIMES_SEP = ",";
  class Client {
    public String ipAddress = "";
    public boolean deleteFlag = false;
    public String userId = "";
    public String info = "";
    public String times = "";
  }
  private ArrayList<Client> clientList;
  private Snapshot snapshot;
  private UserList userList;

  public ClientList(Snapshot _snapshot) throws Exception{
    try{
      snapshot = _snapshot;
      clientList = new ArrayList<Client>();
      userList = new UserList(snapshot.ini.getUserListFilePath());
    }catch(Exception e){
      C.throwException("ClientList: ClientList:", e);
    }
  }

  public String getIpAddressByUserId(String userId){
    for(int i = 0; i < clientList.size(); i++){
      Client cl = clientList.get(i);
      if(userId.equals(cl.userId))
        return cl.ipAddress;
    }
    return "";
  }

  public String[] getUserList(){
    int n = clientList.size();
    if(n == 0)
      return null;
    String[] sa = new String[n];
    for(int i = 0; i < n; i++){
      Client cl = clientList.get(i);
      String s = cl.userId + FIELD_SEP;
      s += userList.get(cl.userId) + FIELD_SEP;
      s += (cl.deleteFlag ? DELETE_FLAG_TRUE : DELETE_FLAG_FALSE) + FIELD_SEP;
      s += cl.ipAddress + FIELD_SEP;
      s += cl.info + FIELD_SEP;
      s += cl.times;
      sa[i] = s;
    }
    Arrays.sort(sa);
    return sa;
  }

  public void addMessage(String ipAddress, String message){
    int i = getIndexByIpAddress(ipAddress);
    if(i == -1){
      C.e("ClientList: addMessage: 1: " + ipAddress + " " + message);
      return;
    }
    Client cl = clientList.get(i);
    switch(message.charAt(0)){
    case TcpBase.MESSAGE_TYPE_USERID:
      String userId = message.substring(1);
      if(cl.userId == ""){
        cl.userId = userId;
        C.d("ClientList: addMessage: 2: " + ipAddress + " " + message);
      }else{
        if(userId.equals(cl.userId)){
          C.d("ClientList: addMessage: 3: " + ipAddress + " " + message);
        }else{
          cl.deleteFlag = true;
          cl = new Client();
          cl.ipAddress = ipAddress;
          cl.userId = userId;
          clientList.add(cl);
          C.d("ClientList: addMessage: 4: " + ipAddress + " " + message);
        }
      }
      cl.times += TIMES_SEP + C.getCustomDateTimeString(C.CUSTOM_DATE_TIME_HCM) + message.substring(0, 1);
      break;
    case TcpBase.MESSAGE_TYPE_REMOTE_DOWN:
      cl.times += TIMES_SEP + C.getCustomDateTimeString(C.CUSTOM_DATE_TIME_HCM) + message.substring(0, 1);
      C.d("ClientList: addMessage: 5: " + ipAddress + " " + message);
      break;
    }
  }

  public void addClient(String ipAddress){
    if(getIndexByIpAddress(ipAddress) == -1){
      Client cl = new Client();
      cl.ipAddress = ipAddress;
      cl.times += C.getCustomDateTimeString(C.CUSTOM_DATE_TIME_HCM)
        + Character.toString(TcpBase.MESSAGE_TYPE_CONNECT);
      clientList.add(cl);
    }
  }

  int getIndexByIpAddress(String ipAddress){
    int i = -1, n = clientList.size();
    for(i = 0; i < n; i++){
      Client cl = clientList.get(i);
      if(cl.ipAddress.equals(ipAddress) && (cl.deleteFlag == false))
        break;
    }
    if(i >= 0 && i < n)
      return i;
    return -1;
  }

}
