/*
Copyright (C) 2012 Taisuke Kobayashi
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.io.File;
import javax.imageio.ImageIO;

class Server implements Runnable {

  private Snapshot snapshot;
  private ServerCanvas serverCanvas;
  private ServerMulticast serverMulticast;
  private TcpServer tcpServer;
  private boolean sendImageLoopFlag = false;
  private boolean requestSendImageFlag = false;
  ClientList clientList;

  public Server(Snapshot _snapshot) throws Exception{
    try{
      snapshot = _snapshot;
      serverCanvas = new ServerCanvas(snapshot);
      serverMulticast = new ServerMulticast(snapshot);
      clientList = new ClientList(snapshot);
      tcpServer = new TcpServer(snapshot);
    }catch(Exception e){
      C.throwException("Server: Server:", e.toString());
    }
  }

  public boolean getSendImageLoopFlag(){ return sendImageLoopFlag; }
  public boolean getRequestSendImageFlag(){ return requestSendImageFlag; }

  public void cancelSendImage(){
    sendImageLoopFlag = false;
    requestSendImageFlag = false;
    tcpServer.cancelSendImage();
  }

  public void requestSendImage(String userId){
    sendImageLoopFlag = false;
    requestSendImageFlag = true;
    String ipAddress = clientList.getIpAddressByUserId(userId);
    if(ipAddress == ""){
      C.error("Server: requestSendImage: ipAddress == \"\" " + userId);
      return;
    }
    tcpServer.requestSendImage(ipAddress);
    C.d("TcpServer: requestSendImage: " + userId + " " + ipAddress);
  }

  public void sendImage(){
    if(sendImageLoopFlag)
      return;
    sendImageLoopFlag = true;
    new Thread(this).start();
  }

  public void run(){
    try{
      while(sendImageLoopFlag){
        sendImageInThread();
      }
    }catch(Exception e){
      C.log("Server: run:" + C.cr + e.toString());
    }
  }

  void sendImageInThread(){
    try{
      snapshot.capture();
      serverMulticast.sendImage(ServerMulticast.SERVER_SEND_IMAGE, snapshot.captureImage);
    }catch(Exception e){
      C.error("Server: sendImage:" + C.cr + e.toString());
    }
  }

  public ServerCanvas getServerCanvas(){ return serverCanvas; }

  public void captureAndSave(){
    try{
      snapshot.capture();
      writeCaptureFile();
    }catch(Exception e){
      C.error("Server: captureAndSave:" + C.cr + e.toString());
    }
  }

  public void writeCaptureFile() throws Exception{
    String path = "";
    try{
      path = String.format(snapshot.PATH_FORMAT, snapshot.ini.getCaptureDir(),
                           snapshot.ini.getBaseName(), C.getDateTimeString());
      ImageIO.write(snapshot.captureImage, snapshot.IMAGE_TYPE, new File(path));
    }catch(Exception e){
      C.throwException("Server: writeCaptureFile: " + path, e);
    }
  }

}
