/*
Copyright (C) 2012 Taisuke Kobayashi
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.net.Socket;
import java.io.InputStream;
import java.io.OutputStream;

public class TcpClient extends TcpBase {

  final int BUF_SIZE = 256;
  private Socket socket;
  private InputStream is;
  private OutputStream os;
  private byte[] buf;

  public TcpClient(Snapshot _snapshot) throws Exception{
    super(_snapshot);
    try{
      socket = new Socket(snapshot.ini.getUnicastIpForClient(), snapshot.ini.getUnicastPortForClient());
      is = socket.getInputStream();
      os = socket.getOutputStream();
      write(Character.toString(MESSAGE_TYPE_USERID) + snapshot.userId);
      buf = new byte[BUF_SIZE];
      C.d("TcpClient: TcpClient: MESSAGE_TYPE_USERID");
      start();
    }catch(Exception e){
      C.throwException("TcpClient: TcpClient:", e);
    }
  }

  public void run(){
    try{
      while(true){
        int len = is.read(buf, 0, BUF_SIZE);
        String s = new String(buf, 0, len, charset);
        switch(s.charAt(0)){
        case MESSAGE_TYPE_REQUEST_SEND_IMAGE:
          switch(s.charAt(1)){
          case '0': snapshot.client.clientMulticast.setSendImageMode(ClientMulticast.SEND_IMAGE_RAW); break;
          case '1': snapshot.client.clientMulticast.setSendImageMode(ClientMulticast.SEND_IMAGE_PNG); break;
          }
          snapshot.client.sendImage();
          C.d("TcpClient: run: MESSAGE_TYPE_REQUEST_SEND_IMAGE: " + s);
          break;
        case MESSAGE_TYPE_CANCEL_SEND_IMAGE:
          snapshot.client.cancelImage();
          C.d("TcpClient: run: MESSAGE_TYPE_CANCEL_SEND_IMAGE: " + s);
          break;
        }
      }
    }catch(Exception e){
      C.log("TcpClient: run: " + e.toString());
    }
  }

  public void write(String s) throws Exception{
    try{
      byte[] b = s.getBytes(charset);
      os.write(b, 0, b.length);
      os.flush();
    }catch(Exception e){
      C.throwException("TcpClient: write:", e);
    }
  }

}
