import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Iterator;

public class TcpServer extends TcpBase {

  private final int BUF_SIZE = 1024;
  private ByteBuffer buf;
  private Selector selector;
  private String ipAddressForSendImage = "";

  public TcpServer(Snapshot _snapshot) throws Exception{
    super(_snapshot);
    try{
      buf = ByteBuffer.allocate(BUF_SIZE);
      selector = Selector.open();
      ServerSocketChannel ssc = ServerSocketChannel.open();
      ssc.configureBlocking(false);
      ssc.socket().setReuseAddress(true);
      ssc.socket().bind(new InetSocketAddress(snapshot.ini.getUnicastPortForServer()));
      ssc.register(selector, SelectionKey.OP_ACCEPT);
      snapshot.ini.writeUnicastIpPortFileForClient(ssc.socket().getInetAddress().getHostAddress());
      start();
    }catch(Exception e){
      C.throwException("TcpServer: TcpServer:", e);
    }
  }

  public void cancelSendImage(){
    try{
      if(ipAddressForSendImage == "")
        return;
      Iterator it = selector.keys().iterator();
      while(it.hasNext()){
        SelectionKey sk = (SelectionKey)it.next();
        if((sk.readyOps() & SelectionKey.OP_READ) == SelectionKey.OP_READ){
          SocketChannel sc = (SocketChannel)sk.channel();
          if(ipAddressForSendImage.equals(sc.socket().getInetAddress().getHostAddress())){
            String s = Character.toString(TcpBase.MESSAGE_TYPE_CANCEL_SEND_IMAGE);
            if(sc.write(charset.encode(s)) <= 0)
              C.error("TcpServer: cancelSendImage: sc.write: " + s);
            ipAddressForSendImage = "";
            return;
          }
        }
      }
      C.error("TcpServer: cancelSendImage: M悪܂łB");
    }catch(Exception e){
      C.error("TcpServer: cancelSendImage: " + e.toString());
    }
  }

  public void requestSendImage(String ipAddress){
    try{
      Iterator it = selector.keys().iterator();
      while(it.hasNext()){
        SelectionKey sk = (SelectionKey)it.next();
        if((sk.readyOps() & SelectionKey.OP_READ) == SelectionKey.OP_READ){
          SocketChannel sc = (SocketChannel)sk.channel();
          if(ipAddress.equals(sc.socket().getInetAddress().getHostAddress())){
            String s = Character.toString(TcpBase.MESSAGE_TYPE_REQUEST_SEND_IMAGE);
            s += snapshot.ini.getSendImageMode();
            if(sc.write(charset.encode(s)) <= 0)
              C.error("TcpServer: requestSendImage: sc.write: " + s);
            ipAddressForSendImage = ipAddress;
            return;
          }
        }
      }
      C.error("TcpServer: requestSendImage: M悪܂łB");
    }catch(Exception e){
      C.error("TcpServer: requestSendImage: " + e.toString());
    }
  }

  public void run(){
    try{
      while(true){
        while(selector.select() > 0){
          Iterator it = selector.selectedKeys().iterator();
          while(it.hasNext()){
            SelectionKey sk = (SelectionKey)it.next();
            it.remove();
            if(sk.isAcceptable()){
              accept((ServerSocketChannel)sk.channel());
            }else if(sk.isReadable()){
              SocketChannel sc = (SocketChannel)sk.channel();
              if(read(sc) < 0){
                sk.cancel();
                sc.close();
              }
            }
          }
        }
      }
    }catch(Exception e){
      C.log("TcpServer: run:" + C.cr + e.toString());
    }
  }

  void accept(ServerSocketChannel ssc) throws Exception{
    try{
      SocketChannel sc = ssc.accept();
      sc.configureBlocking(false);
      sc.register(selector, SelectionKey.OP_READ);
      String ip = sc.socket().getInetAddress().getHostAddress();
      snapshot.server.clientList.addClient(ip);
      C.d("accept: " + ip);
    }catch(Exception e){
      C.throwException("TcpServer: accept:", e);
    }
  }

  int read(SocketChannel sc){
    String ip = "", msg = "";
    try{
      ip = sc.socket().getInetAddress().getHostAddress();
      buf.clear();
      if(sc.read(buf) < 0)
        return 0;
      buf.flip();
      msg = charset.decode(buf).toString();
      snapshot.server.clientList.addMessage(ip, msg);
    }catch (Exception e){
      C.e("TcpServer: read: ip=" + ip + ": msg=" + msg + ": " + e.toString());
      if(ip != "")
        snapshot.server.clientList.addMessage(ip, Character.toString(TcpBase.MESSAGE_TYPE_REMOTE_DOWN));
      return -1;
    }
    return 0;
  }

}
